@@ -1,4 +1,4 @@
-/*
+/* 
  * mt.c: implementation of mt(1) as part of a Cygwin environment
  *
  * Copyright 1998,1999,2000,2001  Corinna Vinschen,
@@ -40,7 +40,7 @@
 
 #include <windows.h>
 
-static char *SCCSid = "@(#)mt V2.1, Corinna Vinschen, " __DATE__ "\n";
+static char *SCCSid = "@(#)mt V2.3, Corinna Vinschen, " __DATE__ "\n";
 
 char *myname;
 
@@ -74,15 +74,22 @@ struct
   {0x15, "EXB-8500 (RLL 45434 bpi)"},
   {0x16, "MFM 10000 bpi"},
   {0x17, "MFM 42500 bpi"},
-  {0x21, "QIC-5010-DC"},
-  {0x22, "QIC-2GB-DC"},
+  {0x19, "DLTtape III (62500 bpi)"},
+  {0x1a, "DLTtape IV (81633 bpi)"},
+  {0x1b, "DLTtape IV (85937 bpi)"},
+  {0x21, "QIC-5010-DC (SLR 32)"},
+  {0x22, "QIC-2GB-DC (DC 9250)"},
   {0x24, "DDS-2"},
-  {0x26, "QIC-4GB-DC"},
-  {0x30, "MLR3"},
-  {0x32, "ALRF-2"},
-  {0x33, "SLR6"},
-  {0x34, "ALRF-1"},
-  {0x36, "ALRF-6"},
+  {0x26, "QIC-4GB-DC (SLR 5)"},
+  {0x30, "MLR3 (SLR 50)"},
+  {0x32, "ALRF-2 (SLR 7)"},
+  {0x33, "SLR6 (SLR 24)"},
+  {0x34, "ALRF-1 (SLR 40/60/75/100)"},
+  {0x36, "ALRF-6 (SLR 140)"},
+  {0x40, "DLT 1/DLT IV (123090 bpi)"},
+  {0x41, "DLTtape IV (98250 bpi)"},
+  {0x48, "SDLTtape I (133000 bpi)"},
+  {0x49, "SDLTtape I (190000 bpi)"},
   {0x81, "EXB-V23 compressed"},
   {0x8c, "EXB-8505 compressed"},
   {0x90, "EXB-8205 compressed"},
@@ -119,9 +126,98 @@ check_cygwin_api_version (int major, int minor)
   return 1;
 }
 
+struct {
+  DWORD value;
+  char *str;
+} feat[] = {
+  { TAPE_DRIVE_FIXED_BLOCK,		"fixed length blocks" },
+  { TAPE_DRIVE_VARIABLE_BLOCK,		"var length blocks" },
+  { TAPE_DRIVE_SET_BLOCK_SIZE,		"set block size" },
+
+  { TAPE_DRIVE_FIXED,			"fixed partitioning" },
+  { TAPE_DRIVE_SELECT,			"select partitioning" },
+  { TAPE_DRIVE_INITIATOR,		"initiator partitioning" },
+
+  { TAPE_DRIVE_GET_LOGICAL_BLK,		"get logical blockno" },
+  { TAPE_DRIVE_LOGICAL_BLK,		"logical block spacing" },
+  { TAPE_DRIVE_GET_ABSOLUTE_BLK,	"get absolute blockno" },
+  { TAPE_DRIVE_ABSOLUTE_BLK,		"absolute block spacing" },
+  { TAPE_DRIVE_LOG_BLK_IMMED,		"logical block immediate" },
+  { TAPE_DRIVE_ABS_BLK_IMMED,		"absolute block immediate" },
+
+  { TAPE_DRIVE_RELATIVE_BLKS,		"relative block spacing" },
+  { TAPE_DRIVE_REVERSE_POSITION,	"backward spacing" },
+
+  { TAPE_DRIVE_REWIND_IMMEDIATE,	"immediate rewind" },
+  { TAPE_DRIVE_END_OF_DATA,		"fast EOM spacing" },
+
+  { TAPE_DRIVE_SPACE_IMMEDIATE,		"immediate spacing" },
+  { TAPE_DRIVE_WRITE_MARK_IMMED,	"write marks immediate" },
+
+  { TAPE_DRIVE_FILEMARKS,		"filemark spacing" },
+  { TAPE_DRIVE_WRITE_FILEMARKS,		"write filemarks" },
+  { TAPE_DRIVE_WRITE_LONG_FMKS,		"write long filemarks" },
+  { TAPE_DRIVE_WRITE_SHORT_FMKS,	"write short filemarks" },
+
+  { TAPE_DRIVE_REPORT_SMKS,		"report setmarks" },
+  { TAPE_DRIVE_SETMARKS,		"setmark spacing" },
+  { TAPE_DRIVE_WRITE_SETMARKS,		"write setmarks" },
+  { TAPE_DRIVE_SET_REPORT_SMKS,		"set report setmarks" },
+
+  { TAPE_DRIVE_SEQUENTIAL_FMKS,		"sequential filemarks" },
+  { TAPE_DRIVE_SEQUENTIAL_SMKS,		"sequential setmarks" },
+
+  { TAPE_DRIVE_LOAD_UNLOAD,		"load and unload" },
+  { TAPE_DRIVE_LOAD_UNLD_IMMED,		"un/load immediate" },
+
+  { TAPE_DRIVE_LOCK_UNLOCK,		"lock and unlock" },
+  { TAPE_DRIVE_LOCK_UNLK_IMMED,		"un/lock immediate" },
+
+  { TAPE_DRIVE_TENSION,			"tape retension" },
+  { TAPE_DRIVE_TENSION_IMMED,		"retension immediate" },
+
+  { TAPE_DRIVE_ERASE_BOP_ONLY,		"erase on bop only" },
+  { TAPE_DRIVE_ERASE_IMMEDIATE,		"erase immediately" },
+  { TAPE_DRIVE_ERASE_LONG,		"long erase" },
+  { TAPE_DRIVE_ERASE_SHORT,		"short erase" },
+
+  { TAPE_DRIVE_EJECT_MEDIA,		"sw ejects media" },
+  { TAPE_DRIVE_WRITE_PROTECT,		"report write protection" },
+
+  { TAPE_DRIVE_EOT_WZ_SIZE,		"report EOT warn size" },
+  { TAPE_DRIVE_SET_EOT_WZ_SIZE,		"set EOT warn size" },
+
+  { TAPE_DRIVE_PADDING,			"data padding" },
+  { TAPE_DRIVE_SET_PADDING,		"set data padding" },
+
+  { TAPE_DRIVE_TAPE_CAPACITY,		"returns capacity" },
+  { TAPE_DRIVE_TAPE_REMAINING,		"returns remaining" },
+
+  { TAPE_DRIVE_ECC,			"hw error correction" },
+  { TAPE_DRIVE_SET_ECC,			"set hw error correction" },
+
+  { TAPE_DRIVE_COMPRESSION,		"hw compression" },
+  { TAPE_DRIVE_SET_COMPRESSION,		"set compression" },
+  { TAPE_DRIVE_SET_CMP_BOP_ONLY,	"set comp. on bop only" },
+  { TAPE_DRIVE_CLEAN_REQUESTS,		"report cleaning request" },
+
+  { 0,					NULL }
+};
+
+static const char *
+get_feature (int i, struct mtget *get)
+{
+  int ret = ((feat[i].value & TAPE_DRIVE_HIGH_FEATURES)
+	     ? ((get->mt_featureshigh & feat[i].value) != 0)
+	     : ((get->mt_featureslow & feat[i].value) != 0));
+  return ret ? "yes" : "no ";
+}
+
 int
 tape_status (int fd, struct mtget *get, long type)
 {
+  int i;
+
   long cygwin_api_minor =
   	strtol (strstr ((char *) cygwin_internal (CW_GETVERSIONINFO),
 			"api minor: ") + 11, NULL, 10);
@@ -137,6 +233,7 @@ tape_status (int fd, struct mtget *get, long type)
 
   if (has_tape)
     {
+      struct mtpos pos;
       int i;
       char *ti_name = "Generic tape device";
       for (i = 0; mt_tape_info[i].t_name; ++i)
@@ -145,29 +242,105 @@ tape_status (int fd, struct mtget *get, long type)
 	    ti_name = mt_tape_info[i].t_name;
 	    break;
 	  }
-	printf ("drive type       :       %02x (%s)\n", get->mt_type, ti_name);
-      if (get->mt_featureslow & TAPE_DRIVE_TAPE_CAPACITY)
-	printf ("tape capacity    : %8lu KB          ", get->mt_capacity >> 10);
-      if (get->mt_featureslow & TAPE_DRIVE_TAPE_REMAINING)
-	printf ("remaining        : %8lu KB\n", get->mt_remaining >> 10);
-      else if (get->mt_featureslow & TAPE_DRIVE_TAPE_CAPACITY)
-	putc ('\n', stdout);
-      if (get->mt_featureslow
-          & (TAPE_DRIVE_GET_ABSOLUTE_BLK | TAPE_DRIVE_GET_LOGICAL_BLK))
-	printf ("current block    : %8lu             ", get->mt_blkno);
-      if (check_cygwin_api_version (0, 112) >= 0)
-	printf ("active partition : %8lu", get->mt_resid);
-      puts ("");
+      if (type <= 1 && check_cygwin_api_version (0, 113) >= 0)
+        {
+	  int ds = (get->mt_dsreg & MT_ST_DENSITY_MASK) >> MT_ST_DENSITY_SHIFT;
+	  int i;
+	  char *ds_name = "unknown";
+	  printf ("drive type = %s\n", ti_name);
+	  printf ("drive status = %lu\n", get->mt_dsreg);
+	  printf ("sense key error = %ld\n", get->mt_erreg);
+	  printf ("residue count = %ld\n", get->mt_resid);
+	  printf ("file number = %ld\n", get->mt_fileno);
+	  printf ("block number = %ld\n", get->mt_blkno);
+	  printf ("Tape block size %lu bytes. ",
+		  (get->mt_dsreg & MT_ST_BLKSIZE_MASK) >> MT_ST_BLKSIZE_SHIFT);
+	  for (i = 0; densities[i].name; ++i)
+	    if (ds == densities[i].code)
+	      {
+		ds_name = densities[i].name;
+		break;
+	      }
+	  printf ("Density code %02x (%s).\n",
+		  (get->mt_dsreg & MT_ST_DENSITY_MASK) >> MT_ST_DENSITY_SHIFT,
+		  ds_name);
+	}
+      else
+	{
+	  printf ("drive type       :       %02x (%s)\n", get->mt_type, ti_name);
+	  if (get->mt_featureslow & TAPE_DRIVE_TAPE_CAPACITY)
+	    printf ("tape capacity    : %8lu KB          ", get->mt_capacity >> 10);
+	  if (get->mt_featureslow & TAPE_DRIVE_TAPE_REMAINING)
+	    printf ("remaining        : %8lu KB\n", get->mt_remaining >> 10);
+	  else if (get->mt_featureslow & TAPE_DRIVE_TAPE_CAPACITY)
+	    putc ('\n', stdout);
+	  if (check_cygwin_api_version (0, 113) >= 0)
+	    printf ("current file     : %8ld             ", get->mt_fileno);
+	  else
+	    printf ("cur logical block: %8lu", get->mt_blkno);
+	  if (check_cygwin_api_version (0, 112) >= 0)
+	    printf ("active partition : %8lu", get->mt_resid);
+	  puts ("");
+	  if (check_cygwin_api_version (0, 113) >= 0)
+	    {
+	      printf ("current block    : %8ld             ", get->mt_blkno);
+	      if (!ioctl (fd, MTIOCPOS, &pos))
+		printf ("cur logical block: %8ld", pos.mt_blkno);
+	      puts ("");
+	    }
+        }
     }
-
-  if (has_tape && (get->mt_featureslow & TAPE_DRIVE_WRITE_PROTECT))
-    printf ("write protected  :      %s             ",
-	    GMT_WR_PROT (get->mt_gstat) ? "yes" : " no");
-  if (get->mt_featureslow & TAPE_DRIVE_COMPRESSION)
-    printf ("datcompression   :      %s\n",
-	    GMT_HW_COMP (get->mt_gstat) ? " on" : "off");
-  else if (has_tape && (get->mt_featureslow & TAPE_DRIVE_WRITE_PROTECT))
-    putc ('\n', stdout);
+  printf("General status bits on (%x):\n", get->mt_gstat);
+  if (GMT_EOF (get->mt_gstat))
+    printf (" EOF");
+  if (GMT_BOT (get->mt_gstat))
+    printf (" BOT");
+  if (GMT_EOT (get->mt_gstat))
+    printf (" EOT");
+  if (GMT_SM (get->mt_gstat))
+    printf (" SM");
+  if (GMT_EOD (get->mt_gstat))
+    printf (" EOD");
+  if (GMT_WR_PROT (get->mt_gstat))
+    printf (" WR_PROT");
+  if (GMT_ONLINE (get->mt_gstat))
+    printf (" ONLINE");
+  if (GMT_D_6250 (get->mt_gstat))
+    printf (" D_6250");
+  if (GMT_D_1600 (get->mt_gstat))
+    printf (" D_1600");
+  if (GMT_D_800 (get->mt_gstat))
+    printf (" D_800");
+  if (GMT_DR_OPEN (get->mt_gstat))
+    printf (" DR_OPEN");       
+  if (GMT_IM_REP_EN (get->mt_gstat))
+    printf (" IM_REP_EN");
+  if (type > 1)
+    {
+      if (GMT_CLN (get->mt_gstat))
+	printf (" CLN");
+      if (GMT_REP_SM (get->mt_gstat))
+	printf (" REP_SM");
+      if (GMT_PADDING  (get->mt_gstat))
+	printf (" PADDING");
+      if (GMT_HW_ECC  (get->mt_gstat))
+	printf (" HW_ECC");
+      if (GMT_HW_COMP  (get->mt_gstat))
+	printf (" HW_COMP");
+      if (GMT_TWO_FM  (get->mt_gstat))
+	printf (" TWO_FM");
+      if (GMT_FAST_MTEOM  (get->mt_gstat))
+	printf (" FAST_MTEOM");
+      if (GMT_AUTO_LOCK  (get->mt_gstat))
+	printf (" AUTO_LOCK");
+      if (GMT_SYSV  (get->mt_gstat))
+	printf (" SYSV");
+      if (GMT_NOWAIT  (get->mt_gstat))
+	printf (" NOWAIT");
+      if (GMT_ASYNC  (get->mt_gstat))
+	printf (" ASYNC");
+    }
+  puts ("");
   if (type > 1)
     {
       if (get->mt_featureshigh & TAPE_DRIVE_SET_BLOCK_SIZE)
@@ -198,130 +371,24 @@ tape_status (int fd, struct mtget *get, long type)
 		  (get->mt_dsreg & MT_ST_DENSITY_MASK) >> MT_ST_DENSITY_SHIFT,
 		  ds_name);
         }
-      if (get->mt_featureslow & TAPE_DRIVE_ECC)
-	printf ("hw err correction:      %s                  ",
-		GMT_HW_ECC (get->mt_gstat) ? " on" : "off");
-      if (get->mt_featureslow & TAPE_DRIVE_PADDING)
-	printf ("data padding     :      %s\n",
-		GMT_PADDING (get->mt_gstat) ? " on" : "off");
-      else if (get->mt_featureslow & TAPE_DRIVE_ECC)
-	putc ('\n', stdout);
-      if (get->mt_featureslow & TAPE_DRIVE_REPORT_SMKS)
-	printf ("report setmarks  :      %s                  ",
-		GMT_IM_REP_EN (get->mt_gstat) ? " on" : "off");
-
-      if (cygwin_api_minor >= 47 /* That struct member didn't exist prior
-				    to Cygwin's API minor version 47. */
-          && get->mt_featureslow & TAPE_DRIVE_SET_EOT_WZ_SIZE)
+      if (check_cygwin_api_version (0, 47) >= 0
+          && get->mt_featureslow & TAPE_DRIVE_EOT_WZ_SIZE)
         printf ("EOT zone size    : %8lu\n", get->mt_eotwarningzonesize);
-      else if (get->mt_featureslow & TAPE_DRIVE_REPORT_SMKS)
-	putc ('\n', stdout);
+      else
+	puts ("");
     }
+
   if (type > 2)
     {
-      printf ("\nFeatures:\n");
-      printf ("---------\n");
-      printf
-	("hw compression   : %s                  erase on bop only: %s\n",
-	 (get->mt_featureslow & TAPE_DRIVE_COMPRESSION) ? "yes" : " no",
-	 (get->mt_featureslow & TAPE_DRIVE_ERASE_BOP_ONLY) ? "yes" : " no");
-      printf
-	("hw err correction: %s                  erase immediately: %s\n",
-	 (get->mt_featureslow & TAPE_DRIVE_ECC) ? "yes" : " no",
-	 (get->mt_featureslow & TAPE_DRIVE_ERASE_IMMEDIATE) ? "yes" : " no");
-      printf
-	("sw eject media   : %s                  long erase op    : %s\n",
-	 (get->mt_featureslow & TAPE_DRIVE_EJECT_MEDIA) ? "yes" : " no",
-	 (get->mt_featureslow & TAPE_DRIVE_ERASE_LONG) ? "yes" : " no");
-      printf
-	("write protection : %s                  short erase op   : %s\n",
-	 (get->mt_featureslow & TAPE_DRIVE_WRITE_PROTECT) ? "yes" : " no",
-	 (get->mt_featureslow & TAPE_DRIVE_ERASE_SHORT) ? "yes" : " no");
-      printf
-	("fixed length blks: %s                  fixed partitions : %s\n",
-	 (get->mt_featureslow & TAPE_DRIVE_FIXED_BLOCK) ? "yes" : " no",
-	 (get->mt_featureslow & TAPE_DRIVE_FIXED) ? "yes" : " no");
-      printf
-	("var length blks  : %s                  select partitions: %s\n",
-	 (get->mt_featureslow & TAPE_DRIVE_VARIABLE_BLOCK) ? "yes" : " no",
-	 (get->mt_featureslow & TAPE_DRIVE_SELECT) ? "yes" : " no");
-      printf
-	("get abs blockaddr: %s                  returns capacity : %s\n",
-	 (get->mt_featureslow & TAPE_DRIVE_GET_ABSOLUTE_BLK) ? "yes" : " no",
-	 (get->mt_featureslow & TAPE_DRIVE_TAPE_CAPACITY) ? "yes" : " no");
-      printf
-	("get log blockaddr: %s                  returns remaining: %s\n",
-	 (get->mt_featureslow & TAPE_DRIVE_GET_LOGICAL_BLK) ? "yes" : " no",
-	 (get->mt_featureslow & TAPE_DRIVE_TAPE_REMAINING) ? "yes" : " no");
-      printf
-	("data padding     : %s                  report setmarks  : %s\n",
-	 (get->mt_featureslow & TAPE_DRIVE_PADDING) ? "yes" : " no",
-	 (get->mt_featureslow & TAPE_DRIVE_REPORT_SMKS) ? "yes" : " no");
-      printf
-	("rep EOT warn size: %s                  initiator partit.: %s\n",
-	 (get->mt_featureslow & TAPE_DRIVE_EOT_WZ_SIZE) ? "yes" : " no",
-	 (get->mt_featureslow & TAPE_DRIVE_INITIATOR) ? "yes" : " no");
-
-      printf
-	("abs blk position : %s                  abs blk immediate: %s\n",
-	 (get->mt_featureshigh & TAPE_DRIVE_ABSOLUTE_BLK) ? "yes" : " no",
-	 (get->mt_featureshigh & TAPE_DRIVE_ABS_BLK_IMMED) ? "yes" : " no");
-      printf
-	("log blk position : %s                  log blk immediate: %s\n",
-	 (get->mt_featureshigh & TAPE_DRIVE_LOGICAL_BLK) ? "yes" : " no",
-	 (get->mt_featureshigh & TAPE_DRIVE_LOG_BLK_IMMED) ? "yes" : " no");
-      printf
-	("filemark position: %s                  rel blk position : %s\n",
-	 (get->mt_featureshigh & TAPE_DRIVE_FILEMARKS) ? "yes" : " no",
-	 (get->mt_featureshigh & TAPE_DRIVE_RELATIVE_BLKS) ? "yes" : " no");
-      printf
-	("end data position: %s                  reverse position : %s\n",
-	 (get->mt_featureshigh & TAPE_DRIVE_END_OF_DATA) ? "yes" : " no",
-	 (get->mt_featureshigh & TAPE_DRIVE_REVERSE_POSITION) ? "yes" : " no");
-      printf
-	("immediate spacing: %s                  immediate rewind : %s\n",
-	 (get->mt_featureshigh & TAPE_DRIVE_SPACE_IMMEDIATE) ? "yes" : " no",
-	 (get->mt_featureshigh & TAPE_DRIVE_REWIND_IMMEDIATE) ? "yes" : " no");
-      printf
-	("sequential filem.: %s                  sequential setm. : %s\n",
-	 (get->mt_featureshigh & TAPE_DRIVE_SEQUENTIAL_FMKS) ? "yes" : " no",
-	 (get->mt_featureshigh & TAPE_DRIVE_SEQUENTIAL_SMKS) ? "yes" : " no");
-      printf
-	("set block size   : %s                  set compression. : %s\n",
-	 (get->mt_featureshigh & TAPE_DRIVE_SET_BLOCK_SIZE) ? "yes" : " no",
-	 (get->mt_featureshigh & TAPE_DRIVE_SET_COMPRESSION) ? "yes" : " no");
-      printf
-	("set hw compress. : %s                  set data padding : %s\n",
-	 (get->mt_featureshigh & TAPE_DRIVE_SET_ECC) ? "yes" : " no",
-	 (get->mt_featureshigh & TAPE_DRIVE_SET_PADDING) ? "yes" : " no");
-      printf
-	("setmark position : %s                  set report setm. : %s\n",
-	 (get->mt_featureshigh & TAPE_DRIVE_SETMARKS) ? "yes" : " no",
-	 (get->mt_featureshigh & TAPE_DRIVE_SET_REPORT_SMKS) ? "yes" : " no");
-      printf
-	("load and unload  : %s                  un/load immediate: %s\n",
-	 (get->mt_featureshigh & TAPE_DRIVE_LOAD_UNLOAD) ? "yes" : " no",
-	 (get->mt_featureshigh & TAPE_DRIVE_LOAD_UNLD_IMMED) ? "yes" : " no");
-      printf
-	("lock and unlock  : %s                  un/lock immediate: %s\n",
-	 (get->mt_featureshigh & TAPE_DRIVE_LOCK_UNLOCK) ? "yes" : " no",
-	 (get->mt_featureshigh & TAPE_DRIVE_LOCK_UNLK_IMMED) ? "yes" : " no");
-      printf
-	("tape retension   : %s                  retens. immediate: %s\n",
-	 (get->mt_featureshigh & TAPE_DRIVE_TENSION) ? "yes" : " no",
-	 (get->mt_featureshigh & TAPE_DRIVE_TENSION_IMMED) ? "yes" : " no");
-      printf
-	("write filemarks  : %s                  write setmarks   : %s\n",
-	 (get->mt_featureshigh & TAPE_DRIVE_WRITE_FILEMARKS) ? "yes" : " no",
-	 (get->mt_featureshigh & TAPE_DRIVE_WRITE_SETMARKS) ? "yes" : " no");
-      printf
-	("write long filem.: %s                  write marks immed: %s\n",
-	 (get->mt_featureshigh & TAPE_DRIVE_WRITE_LONG_FMKS) ? "yes" : " no",
-	 (get->mt_featureshigh & TAPE_DRIVE_WRITE_MARK_IMMED) ? "yes" : " no");
-      printf
-	("write shrt filem.: %s                  set EOT warn size: %s\n",
-	 (get->mt_featureshigh & TAPE_DRIVE_WRITE_SHORT_FMKS) ? "yes" : " no",
-	 (get->mt_featureslow & TAPE_DRIVE_SET_EOT_WZ_SIZE) ? "yes" : " no");
+      printf ("Drive(r) Capabilities:\n");
+      printf ("----------------------\n");
+      for (i = 0; feat[i].value; ++i)
+        {
+	  printf ("%-27.27s: %s        ", feat[i].str, get_feature (i, get));
+	  if (i % 2)
+	    puts ("");
+	}
+      puts ("");
     }
   return NO_ERROR;
 }
@@ -374,7 +441,7 @@ main (int argc, char *argv[])
   if (optind >= argc)
     return usage ();
   cmd = argv[optind++];
-  if (optind < argc && (count = atol (argv[optind])) < 0)
+  if (optind < argc && (count = strtol (argv[optind], NULL, 0)) < 0)
     return errprintf (1, "count must be positive.");
 
   if ((fd = open (path, O_RDWR)) < 0)
@@ -392,7 +459,7 @@ main (int argc, char *argv[])
   if (!strcmp (cmd, "status"))
     {
       struct mtget get;
-      if (ioctl (fd, MTIOCGET, &get) < 0)
+      if (ioctl (fd, MTIOCGET, &get) < 0 && errno != ENOMEDIUM)
 	ret = errprintf (2, "%s: %s", path, strerror (errno));
       else
         tape_status (fd, &get, count);
@@ -550,7 +617,7 @@ main (int argc, char *argv[])
       if (ioctl (fd, MTIOCTOP, &op) < 0)
 	ret = errprintf (2, "%s: %s", path, strerror (errno));
     }
-  /* set drive buffer size */
+  /* set drive buffer size or flags */
   else if (!strcmp (cmd, "drvbuffer"))
     {
       op.mt_op = MTSETDRVBUFFER;
@@ -560,7 +627,11 @@ main (int argc, char *argv[])
   /* set driver options */
   else if (!strcmp (cmd, "stoptions"))
     {
-      ret = errprintf (1, "operation '%s' not implemented", cmd);
+      op.mt_op = MTSETDRVBUFFER;
+      op.mt_count &= ~MT_ST_OPTIONS;
+      op.mt_count |= MT_ST_BOOLEANS;
+      if (ioctl (fd, MTIOCTOP, &op) < 0)
+	ret = errprintf (2, "%s: %s", path, strerror (errno));
     }
   /* set write threshold */
   else if (!strcmp (cmd, "stwrthreshold"))
@@ -595,9 +666,17 @@ main (int argc, char *argv[])
   /* get/set compression */
   else if (!strcmp (cmd, "datcompression"))
     {
-      op.mt_op = MTCOMPRESSION;
-      if (ioctl (fd, MTIOCTOP, &op) < 0)
-	ret = errprintf (2, "%s: %s", path, strerror (errno));
+      struct mtget get;
+
+      if (count != 1)
+        {
+	  op.mt_op = MTCOMPRESSION;
+	  if (ioctl (fd, MTIOCTOP, &op) < 0)
+	    ret = errprintf (2, "%s: %s", path, strerror (errno));
+	}
+      if (!ioctl (fd, MTIOCGET, &get) || errno == ENOMEDIUM)
+	printf ("Compression %s.\n", GMT_HW_COMP (get.mt_gstat)
+				     ? "on" : "off");
     }
   else if (!strcmp (cmd, "mkpart"))
     {
